package ru.dbotthepony.kstarbound.client.gl.shader

import ru.dbotthepony.kstarbound.client.StarboundClient
import ru.dbotthepony.kstarbound.client.gl.vertex.GeometryType
import ru.dbotthepony.kstarbound.client.gl.vertex.StreamVertexBuilder
import ru.dbotthepony.kstarbound.client.gl.vertex.VertexAttributes
import ru.dbotthepony.kvector.api.IStruct4f
import ru.dbotthepony.kvector.arrays.Matrix3f
import ru.dbotthepony.kvector.vector.RGBAColor

private fun shaders(name: String): List<GLShader> {
	val client = StarboundClient.current()
	return listOf(client.internalVertex("shaders/$name.vsh"), client.internalFragment("shaders/$name.fsh"))
}

private fun shaders1(name: String): List<GLShader> {
	val client = StarboundClient.current()
	return listOf(client.internalVertex("shaders/vertex/$name.vsh"), client.internalFragment("shaders/fragment/$name.fsh"))
}

class FontProgram : GLShaderProgram(shaders1("font"), VertexAttributes.POSITION_UV), GLShaderProgram.Regular {
	override var viewMatrix: Matrix3f by F3x3Uniform("viewMatrix", true)
	override var worldMatrix: Matrix3f by F3x3Uniform("worldMatrix", true)
	override var modelMatrix: Matrix3f by F3x3Uniform("modelMatrix", true)
	override var colorMultiplier: IStruct4f by F4Uniform("colorMultiplier", true)
	var texture by IUniform("texture0")

	val builder = StreamVertexBuilder(attributes, GeometryType.QUADS)

	init {
		viewMatrix = Matrix3f.identity()
		modelMatrix = Matrix3f.identity()
		colorMultiplier = RGBAColor.WHITE
	}
}

class GLLiquidProgram : GLShaderProgram(shaders("liquid"), VertexAttributes.POSITION) {
	var baselineColor by F4Uniform("baselineColor")
	var transform by F3x3Uniform("transform")
	val builder = StreamVertexBuilder(attributes, GeometryType.QUADS)
}

class GLPrograms {
	val position = UberShader.Builder().build()
	val positionTexture = UberShader.Builder().withTexture().build()
	val positionTextureLightmap = UberShader.Builder().withTexture().withLightMap().build()
	val positionColor = UberShader.Builder().withColor().build()
	val tile = UberShader.Builder().withTexture().withHueShift().withLightMap().build()
	val font = FontProgram()
	val liquid = GLLiquidProgram()
}
